@@ -29,17 +29,18 @@ Use :py:meth:`~postgres.Postgres.run` to run SQL statements:
     >>> db.run("INSERT INTO foo VALUES ('baz')")
     >>> db.run("INSERT INTO foo VALUES ('buz')")
 
-Use :py:meth:`~postgres.Postgres.one` to fetch exactly one result:
-
-    >>> db.one("SELECT * FROM foo WHERE bar='baz'")
-    {'bar': 'baz'}
-
-
 Use :py:meth:`~postgres.Postgres.all` to fetch all results:
 
     >>> db.all("SELECT * FROM foo ORDER BY bar")
     [{'bar': 'baz'}, {'bar': 'buz'}]
 
+Use :py:meth:`~postgres.Postgres.one_or_zero` to fetch one result or
+:py:class:`None`:
+
+    >>> db.one_or_zero("SELECT * FROM foo WHERE bar='baz'")
+    {'bar': 'baz'}
+    >>> db.one_or_zero("SELECT * FROM foo WHERE bar='blam'")
+
 
 Bind Parameters
 +++++++++++++++
@@ -63,11 +64,11 @@ Context Managers
 ++++++++++++++++
 
 Eighty percent of your database usage should be covered by the simple
-:py:meth:`~postgres.Postgres.run`, :py:meth:`~postgres.Postgres.one`,
-:py:meth:`~postgres.Postgres.all` API introduced above. For the other 20%,
-:py:mod:`postgres` provides context managers for working at increasingly lower
-levels of abstraction. The lowest level of abstraction in :py:mod:`postgres` is
-a :py:mod:`psycopg2` `connection pool
+:py:meth:`~postgres.Postgres.run`, :py:meth:`~postgres.Postgres.all`,
+:py:meth:`~postgres.Postgres.one_or_zero` API introduced above. For the other
+20%, :py:mod:`postgres` provides context managers for working at increasingly
+lower levels of abstraction. The lowest level of abstraction in
+:py:mod:`postgres` is a :py:mod:`psycopg2` `connection pool
 <http://initd.org/psycopg/docs/pool.html>`_ that we configure and manage for
 you. Everything in :py:mod:`postgres`, both the simple API and the context
 managers, uses this connection pool.
@@ -162,7 +163,7 @@ from psycopg2.extras import RealDictCursor
 from psycopg2.pool import ThreadedConnectionPool as ConnectionPool
 
 
-__version__ = '1.0.1'
+__version__ = '1.1.0'
 
 
 # A Helper
@@ -190,14 +191,25 @@ def url_to_dsn(url):
 # Exceptions
 # ==========
 
-class NotOne(Exception):
-    def __init__(self, rowcount):
-        self.rowcount = rowcount
-    def __str__(self):
-        return "Got {0} rows instead of 1.".format(self.rowcount)
+class OutOfBounds(Exception):
 
-class TooFew(NotOne): pass
-class TooMany(NotOne): pass
+    def __init__(self, n, lo, hi):
+        self.n = n
+        self.lo = lo
+        self.hi = hi
+
+    def __str__(self):
+        msg = "Got {n} rows; expecting "
+        if self.lo == self.hi:
+            msg += "exactly {lo}."
+        elif self.hi - self.lo == 1:
+            msg += "{lo} or {hi}."
+        else:
+            msg += "between {lo} and {hi} (inclusive)."
+        return msg.format(**self.__dict__)
+
+class TooFew(OutOfBounds): pass
+class TooMany(OutOfBounds): pass
 
 
 # The Main Event
@@ -237,19 +249,20 @@ class Postgres(object):
     :py:class:`NamedTupleCursor`.
 
     The names in our simple API, :py:meth:`~postgres.Postgres.run`,
-    :py:meth:`~postgres.Postgres.one`, and :py:meth:`~postgres.Postgres.all`,
-    were chosen to be short and memorable, and to not conflict with the DB-API
-    2.0 :py:meth:`execute`, :py:meth:`fetchone`, and :py:meth:`fetchall`
-    methods, which have slightly different semantics (under DB-API 2.0 you call
-    :py:meth:`execute` on a cursor and then call one of the :py:meth:`fetch*`
-    methods on the same cursor to retrieve rows; with our simple API there is
-    no second :py:meth:`fetch` step). See `this ticket`_ for more of the
-    rationale behind these names. The context managers on this class are named
-    starting with :py:meth:`get_` to set them apart from the simple-case API.
-    Note that when working inside a block under one of the context managers,
-    you're using DB-API 2.0 (:py:meth:`execute` + :py:meth:`fetch*`), not our
-    simple API (:py:meth:`~postgres.Postgres.run` /
-    :py:meth:`~postgres.Postgres.one` / :py:meth:`~postgres.Postgres.all`).
+    :py:meth:`~postgres.Postgres.all`, and
+    :py:meth:`~postgres.Postgres.one_or_zero`, were chosen to be short and
+    memorable, and to not conflict with the DB-API 2.0 :py:meth:`execute`,
+    :py:meth:`fetchone`, and :py:meth:`fetchall` methods, which have slightly
+    different semantics (under DB-API 2.0 you call :py:meth:`execute` on a
+    cursor and then call one of the :py:meth:`fetch*` methods on the same
+    cursor to retrieve rows; with our simple API there is no second
+    :py:meth:`fetch` step). See `this ticket`_ for more of the rationale behind
+    these names. The context managers on this class are named starting with
+    :py:meth:`get_` to set them apart from the simple-case API.  Note that when
+    working inside a block under one of the context managers, you're using
+    DB-API 2.0 (:py:meth:`execute` + :py:meth:`fetch*`), not our simple API
+    (:py:meth:`~postgres.Postgres.run` / :py:meth:`~postgres.Postgres.one` /
+    :py:meth:`~postgres.Postgres.all`).
 
     .. _this ticket: https://github.com/gittip/postgres.py/issues/16
 
@@ -272,6 +285,7 @@ class Postgres(object):
             raise ValueError("strict_one must be True, False, or None.")
         self.strict_one = strict_one
 
+
     def run(self, sql, parameters=None):
         """Execute a query and discard any results.
 
@@ -288,52 +302,9 @@ class Postgres(object):
         with self.get_cursor() as cursor:
             cursor.execute(sql, parameters)
 
-    def one(self, sql, parameters=None, strict=None):
-        """Execute a query and return a single result.
-
-        :param unicode sql: the SQL statement to execute
-        :param parameters: the bind parameters for the SQL statement
-        :type parameters: dict or tuple
-        :param strict: whether to raise when there isn't exactly one result
-        :type strict: :py:class:`bool`
-        :returns: a single row or :py:const:`None`
-        :raises: :py:exc:`~postgres.TooFew` or :py:exc:`~postgres.TooMany`
-
-        By default, :py:attr:`strict` ends up evaluating to :py:class:`True`,
-        in which case we raise :py:exc:`~postgres.TooFew` or
-        :py:exc:`~postgres.TooMany` if the number of rows returned isn't
-        exactly one. You can override this behavior per-call with the
-        :py:attr:`strict` argument here, or globally by passing
-        :py:attr:`strict_one` to the :py:class:`~postgres.Postgres`
-        constructor. If you use both, the :py:attr:`strict` argument here wins.
-
-        >>> row = db.one("SELECT * FROM foo WHERE bar='baz'")
-        >>> print(row["bar"])
-        baz
-
-        """
-        if strict not in (True, False, None):
-            raise ValueError("strict must be True, False, or None.")
-
-        if strict is None:
-            if self.strict_one is None:
-                strict = True               # library default
-            else:
-                strict = self.strict_one    # user default
-
-        with self.get_cursor() as cursor:
-            cursor.execute(sql, parameters)
-
-            if strict:
-                if cursor.rowcount < 1:
-                    raise TooFew(cursor.rowcount)
-                elif cursor.rowcount > 1:
-                    raise TooMany(cursor.rowcount)
-
-            return cursor.fetchone()
 
     def all(self, sql, parameters=None):
-        """Execute a query and return all resulting rows.
+        """Execute a query and return all results.
 
         :param unicode sql: the SQL statement to execute
         :param parameters: the bind parameters for the SQL statement
@@ -359,15 +330,103 @@ class Postgres(object):
         return self.all(*a, **kw)
 
 
+    def one_or_zero(self, sql, parameters=None):
+        """Execute a query and return a single result or :py:class:`None`.
+
+        :param unicode sql: the SQL statement to execute
+        :param parameters: the bind parameters for the SQL statement
+        :type parameters: dict or tuple
+        :returns: a single row or :py:const:`None`
+        :raises: :py:exc:`~postgres.TooFew` or :py:exc:`~postgres.TooMany`
+
+        Use this for the common case where there should only be one record, but
+        it may not exist yet.
+
+        >>> row = db.one_or_zero("SELECT * FROM foo WHERE bar='blam'")
+        >>> if row is None:
+        ...     print("No blam yet.")
+        ...
+        No blam yet.
+
+        """
+        return self._some(sql, parameters, 0, 1)
+
+
+    def one(self, sql, parameters=None, strict=None):
+
+        # I'm half-considering dropping this. Now that one_or_zero exists, this
+        # is really only useful for what should really be called db.first, and
+        # in that case why aren't you using a LIMIT 1?
+
+        """Execute a query and return a single result.
+
+        :param unicode sql: the SQL statement to execute
+        :param parameters: the bind parameters for the SQL statement
+        :type parameters: dict or tuple
+        :param strict: whether to raise when there isn't exactly one result
+        :type strict: :py:class:`bool`
+        :returns: a single row or :py:const:`None`
+        :raises: :py:exc:`~postgres.TooFew` or :py:exc:`~postgres.TooMany`
+
+        By default, :py:attr:`strict` ends up evaluating to :py:class:`True`,
+        in which case we raise :py:exc:`postgres.TooFew` or
+        :py:exc:`postgres.TooMany` if the number of rows returned isn't exactly
+        one (both are subclasses of :py:exc:`postgres.OutOfBounds`). You can
+        override this behavior per-call with the :py:attr:`strict` argument
+        here, or globally by passing :py:attr:`strict_one` to the
+        :py:class:`~postgres.Postgres` constructor. If you use both, the
+        :py:attr:`strict` argument here wins. If you pass :py:class:`False`
+        for :py:attr:`strict`, then we return :py:class:`None` if there are no
+        results, and the first if there is more than one.
+
+        >>> row = db.one("SELECT * FROM foo WHERE bar='baz'")
+        >>> print(row["bar"])
+        baz
+
+        """
+        if strict not in (True, False, None):
+            raise ValueError("strict must be True, False, or None.")
+
+        if strict is None:
+            if self.strict_one is None:
+                strict = True               # library default
+            else:
+                strict = self.strict_one    # user default
+
+        if strict:
+            out = self._some(sql, parameters, 1, 1)
+        else:
+            with self.get_cursor() as cursor:
+                cursor.execute(sql, parameters)
+                out = cursor.fetchone()
+        return out
+
+
+    def _some(self, sql, parameters=None, lo=0, hi=1):
+
+        # This is undocumented (and largely untested) because I think it's a
+        # rare case where this is wanted directly. It's here to make one and
+        # one_or_zero DRY. Help yourself to it now that you've found it. :^)
+
+        with self.get_transaction() as txn:
+            txn.execute(sql, parameters)
+
+            if txn.rowcount < lo:
+                raise TooFew(txn.rowcount, lo, hi)
+            elif txn.rowcount > hi:
+                raise TooMany(txn.rowcount, lo, hi)
+
+            return txn.fetchone()
+
+
     def get_cursor(self, *a, **kw):
         """Return a :py:class:`~postgres.CursorContextManager` that uses our
         connection pool.
 
-        This is what :py:meth:`~postgres.Postgres.run`,
-        :py:meth:`~postgres.Postgres.one`, and
-        :py:meth:`~postgres.Postgres.all` use under the hood. You might
-        use it if you want to access `cursor attributes
-        <http://initd.org/psycopg/docs/cursor.html>`_, for example.
+        This gets you a cursor with :py:attr:`autocommit` turned on on its
+        connection. The context manager closes the cursor when the block ends.
+
+        Use this when you want a simple cursor.
 
         >>> with db.get_cursor() as cursor:
         ...     cursor.execute("SELECT * FROM foo")
@@ -382,11 +441,15 @@ class Postgres(object):
         """Return a :py:class:`~postgres.TransactionContextManager` that uses
         our connection pool.
 
+        This gets you a cursor with :py:attr:`autocommit` turned off on its
+        connection. If your code block inside the :py:obj:`with` statement
+        raises an exception, the transaction will be rolled back. Otherwise,
+        it'll be committed. The context manager closes the cursor when the
+        block ends.
+
         Use this when you want a series of statements to be part of one
         transaction, but you don't need fine-grained control over the
-        transaction. If your code block inside the :py:obj:`with` statement
-        raises an exception, the transaction will be rolled back. Otherwise,
-        it'll be committed.
+        transaction.
 
         >>> with db.get_transaction() as txn:
         ...     txn.execute("SELECT * FROM foo")
@@ -452,7 +515,7 @@ class CursorContextManager(object):
     arguments to our constructor are passed through to the cursor constructor.
     The :py:class:`~postgres.Connection` underlying the cursor is checked
     out of the connection pool when the block starts, and checked back in when
-    the block ends.
+    the block ends. Also when the block ends, the cursor is closed.
 
     """
 
@@ -466,11 +529,13 @@ class CursorContextManager(object):
         """Get a connection from the pool.
         """
         self.conn = self.pool.getconn()
-        return self.conn.cursor(*self.a, **self.kw)
+        self.cursor = self.conn.cursor(*self.a, **self.kw)
+        return self.cursor
 
     def __exit__(self, *exc_info):
         """Put our connection back in the pool.
         """
+        self.cursor.close()
         self.pool.putconn(self.conn)
 
 
@@ -485,8 +550,9 @@ class TransactionContextManager(object):
     cursor is checked out of the connection pool and :py:attr:`autocommit` is
     set to :py:const:`False`. If the block raises an exception, the
     :py:class:`~postgres.Connection` is rolled back. Otherwise it's committed.
-    In either case, :py:attr:`autocommit` is restored to :py:const:`True` and
-    the :py:class:`~postgres.Connection` is put back in the pool.
+    In either case, the cursor is closed, :py:attr:`autocommit` is restored to
+    :py:const:`True`, and the :py:class:`~postgres.Connection` is put back in
+    the pool.
 
     """
 
@@ -501,7 +567,8 @@ class TransactionContextManager(object):
         """
         self.conn = self.pool.getconn()
         self.conn.autocommit = False
-        return self.conn.cursor(*self.a, **self.kw)
+        self.cursor = self.conn.cursor(*self.a, **self.kw)
+        return self.cursor
 
     def __exit__(self, *exc_info):
         """Put our connection back in the pool.
@@ -510,6 +577,7 @@ class TransactionContextManager(object):
             self.conn.commit()
         else:
             self.conn.rollback()
+        self.cursor.close()
         self.conn.autocommit = True
         self.pool.putconn(self.conn)
 
